package genFormulas

object Nat {

  abstract sealed class nat
  final case class zero_nat() extends nat
  final case class Suc(a: nat) extends nat

} /* object Nat */

object Lista {

  def map[A, B](f: A => B, x1: List[A]): List[B] = (f, x1) match {
    case (f, Nil)        => Nil
    case (f, x21 :: x22) => f(x21) :: map[A, B](f, x22)
  }

} /* object Lista */

object tp4bis {

  abstract sealed class variable
  final case class A() extends variable
  final case class B() extends variable
  final case class C() extends variable

  abstract sealed class formula
  final case class Var(a: variable) extends formula
  final case class And(a: (formula, formula)) extends formula
  final case class Or(a: (formula, formula)) extends formula

  def allVars: List[variable] = List(A(), B(), C())

  def base1: List[formula] =
    Lista.map[variable, formula](((a: variable) => Var(a)), allVars)

  def allCombinationsOneTR(
      uu: formula,
      x1: List[formula],
      acc: List[formula]
  ): List[formula] =
    (uu, x1, acc) match {
      case (uu, Nil, acc) => acc
      case (x, e :: r, acc) =>
        allCombinationsOneTR(x, r, Or((x, e)) :: And((x, e)) :: acc)
    }

  def allCombinationsTR(
      x0: List[formula],
      uu: List[formula],
      acc: List[formula]
  ): List[formula] =
    (x0, uu, acc) match {
      case (Nil, uu, acc) => acc
      case (e :: r, l, acc) =>
        allCombinationsTR(r, l, allCombinationsOneTR(e, l, acc))
    }

  def genFormulasFullTR(x0: Nat.nat, acc: List[formula]): List[formula] =
    (x0, acc) match {
      case (Nat.zero_nat(), acc) => acc
      case (Nat.Suc(x), acc) => {
        val a = allCombinationsTR(acc, acc, acc): (List[formula]);
        genFormulasFullTR(x, a)
      }
    }

} /* object tp4bis */
